#include <me218_c32.h>
#include <stdio.h>
#include <timers12.h>
#include <ses.h>
#include "PWMS12.h"
#include "BITDEFS.h"
#include "ADS12.h"

/*---------------- Module Defines ---------------------------*/
//The wheel speeds when the robot is cruising around the perimeter
#define HIGHCRUISINGSPEED 10
#define LOWCRUISINGSPEED 9 
#define ONESEC 250
#define LIGHT_HYSTERESIS_CONSTANT 25
#define BEACON_READ_PERIOD ONESEC/1
#define DOORLOCK_WAIT_TIME ONESEC/1
#define FIRST_ORIENTATION_WAIT_TIME ONESEC*4
#define SECOND_ORIENTATION_WAIT_TIME ONESEC
#define INITIAL_WAIT_TIME 3*ONESEC
#define INITIAL_BEACON_WAIT_TIME ONESEC
#define FOUND_BEACON_COUNT_NUMBER 10
#define LIGHT_SENSOR_THRESHOLD 575
/*---------------- Function Prototypes ----------------------*/
unsigned char TestForTmr0(EVENT_PARAM);
void RespToTmr0(SERVICE_PARAM);
unsigned char TestForTmr1(EVENT_PARAM);
void RespToTmr1(SERVICE_PARAM);
unsigned char TestForTmr4(EVENT_PARAM);
void RespToTmr4(SERVICE_PARAM);
unsigned char TestForTmr5(EVENT_PARAM);
void RespToTmr5(SERVICE_PARAM);

unsigned char TestForBeacon(EVENT_PARAM);
void RespToBeacon(SERVICE_PARAM);
unsigned char TestForKey(EVENT_PARAM);
void RespToKey(SERVICE_PARAM);
void RespToBeacon(SERVICE_PARAM);
void EnterOrientationState1();
void EnterOrientationState2();
void EnterWallTravelingState();
void EnterWaitingState();
void SetMotor1Speed(int speed);
void SetMotor2Speed(int speed);
void SetMotor1CW();
void SetMotor1CCW();
void SetMotor2CW();
void SetMotor2CCW();

/*---------------- Module Code -----------------------------*/

void main (void)
{
  /* initialize the SES module to operate in round robin mode, with no
  timer delay */
  SES_Init(SES_ROUND_ROBIN);
  TMRS12_Init(TMRS12_RATE_4MS);      //initialize timer module

  /* register each pair of Events and Services routines*/
  SES_Register(TestForTmr0, RespToTmr0);
  SES_Register(TestForTmr1, RespToTmr1);
  SES_Register(TestForTmr4, RespToTmr4);
  SES_Register(TestForTmr5, RespToTmr5);  
  //SES_Register(TestForBeacon, RespToBeacon);
  //SES_Register(TestForKey,RespToKey);
  
  /*Initialize ports 0-2 as PWM ports*/
  PWMS12_Init();
  
  /*Initialize T ports 5 and 6 to be digital outputs for the h-bridge*/
  DDRT |= BIT5HI;//Initialize PortT5 to be an output           
  PTT |= BIT5HI;//set the motor1 to turn clockwise 
  DDRT |= BIT6HI;//Initialize PortT6 to be an output           
  PTT |= BIT6HI;//set the motor2 to turn clockwise 
  
  /*Initialize M port 0 and 1to be an output*/
  DDRM |= BIT0HI;//Initialize PortM0 to be an output           
  PTM |= BIT0HI;//set the motor to be retracting
  DDRM |= BIT1HI;//Initialize PortM1 to be an output           
  PTM |= BIT1HI;//enable the motor
  
  /*Initialize M port 4 to control resetting the beacon sensing circuit*/
  DDRM |= BIT4HI;//Initialize PortM0 to be an output           
  //reset the beacon sensing circuit
  PTM |= BIT4HI;
  PTM &= BIT4LO;
  
  /*Initialize AD port 0 to be an analog input*/
  if(ADS12_Init("OOOOOOOA")==ADS12_OK){
    printf("successfully initiated ADS12 \r\n");
  }else {
    printf("ADS12 failed\r\n");
    return;
  }  
    
  //begin orientation routine
  //EnterOrientationState1();
  //begin waiting state
  EnterWaitingState(); 
  //EnterWallTravelingState();
 
  
  
  //Initialize timer 1 to regulate doorlock motor
  TMRS12_InitTimer(1, DOORLOCK_WAIT_TIME);  
  //Initialize timer 2 to control beacon sensing period
  TMRS12_InitTimer(2, BEACON_READ_PERIOD);
  //Initialize timer 3 to set a wait time before starting beacon sensing
  TMRS12_InitTimer(3, INITIAL_BEACON_WAIT_TIME);  
  //Initialize timer 7 to set 2 minute max time 
  TMRS12_InitTimer(7, ONESEC*120); 
                      
  /* enter an infinite loop and handle the events */
	while (1) {	
		SES_HandleEvents();
	}
}

//********************************************************************
//State functions
//********************************************************************/
void EnterWaitingState(){
  //Initialize timer 5 to wait after program starts
  TMRS12_InitTimer(5,INITIAL_WAIT_TIME);  
  //disable doorlock motors 1 and 2
  PTM &= BIT0LO;
  PTM &= BIT1LO;    
  //disable wheel motors 1 and 2
  PTM &= BIT0LO;
  PTM &= BIT1LO;  
  
}
void EnterWallTravelingState(){
  SetMotor1CW();
  SetMotor2CW();
  SetMotor1Speed(HIGHCRUISINGSPEED);
  SetMotor2Speed(LOWCRUISINGSPEED);
}

void EnterOrientationState1(){
  TMRS12_InitTimer(0, FIRST_ORIENTATION_WAIT_TIME);  
  SetMotor1CCW();
  SetMotor2CW();
  SetMotor1Speed(5);
  SetMotor2Speed(5);    
}

void EnterOrientationState2(){
  TMRS12_InitTimer(4, SECOND_ORIENTATION_WAIT_TIME);  
  SetMotor1CW();
  SetMotor2CCW();
  SetMotor1Speed(5);
  SetMotor2Speed(5);    
}

//********************************************************************
//Motor functions
//********************************************************************/

void SetMotor1Speed(int speed){
  if(speed>10){
    printf("Error - motor requested motor speed is greater than 10");
    speed = 10;
  }
  PWMS12_SetDuty(10*speed, 0);    
}
void SetMotor2Speed(int speed){
  if(speed>10){
    printf("Error - motor requested motor speed is greater than 10");
    speed = 10;
  }
  PWMS12_SetDuty(10*speed, 1);    
}
void SetMotor1CW(){
  PTT |= BIT5HI;//set the motor1 to turn clockwise   
}

void SetMotor1CCW(){
  PTT &= BIT5LO;//set the motor1 to turn counter clockwise   
}
void SetMotor2CW(){
  PTT |= BIT6HI;//set the motor1 to turn clockwise   
}

void SetMotor2CCW(){
  PTT &= BIT6LO;//set the motor1 to turn counter clockwise   
}

void SetDoor1CW(){
  PTM |= BIT0HI;//set the motor1 to turn clockwise   
}

void SetDoor1CCW(){
  PTM &= BIT0LO;//set the motor1 to turn counter clockwise   
}
void SetDoor2CW(){
  PTM |= BIT1HI;//set the motor1 to turn clockwise   
}

void SetDoor2CCW(){
  PTM &= BIT1LO;//set the motor1 to turn counter clockwise   
}



//********************************************************************
//Event and Service Functions 
//********************************************************************/

unsigned char TestForBeacon(EVENT_PARAM)
{  	
  if(TMRS12_IsTimerExpired(2)) {
    //restart the timer
    TMRS12_InitTimer(2, ONESEC/100);                    
    
    //read the light level
    lightLevel = ADS12_ReadADPin(0);
    if(printCount > 50){
      printf("[Port AD0, maxLightLevel] = [%d,%d] \n\r",lightLevel,maxLightLevel);
      printCount = 0;
    }else{
      printCount++;
    }
    
    //clear the capacitor from the beacon sensor resetting the peak value
    PTM |= BIT4HI;
    PTM &= BIT4LO;
    
    //update maxLightLevel or check if light level has started to drop  
    if(lightLevel>LIGHT_SENSOR_THRESHOLD){
      printf("BEACON FOUND [lightLevel,maxLightLevel] = [%d, %d]\r\n",lightLevel,maxLightLevel); 
      return 1;
    }
  }
  return 0;                 
}

void RespToBeacon(SERVICE_PARAM)
{                       
  EnterWallTravelingState();
  TMRS12_ClearTimerExpired(0);
}


unsigned char TestForTmr0(EVENT_PARAM)
{
  if(TMRS12_IsTimerExpired(0)) {
   printf("timer 0 expired \r\n");
   return 1;
  }
  return 0;
}

void RespToTmr0(SERVICE_PARAM)
{                       
  EnterOrientationState2();
  TMRS12_ClearTimerExpired(0);
}

unsigned char TestForTmr1(EVENT_PARAM)
{
  if(TMRS12_IsTimerExpired(1)) {
   printf("timer 1 expired \r\n");
   return 1;
  }
  return 0;
}

void RespToTmr1(SERVICE_PARAM)
{                       
  PTM &= BIT1LO;//disable the motor    
  TMRS12_ClearTimerExpired(1);
}

unsigned char TestForTmr4(EVENT_PARAM)
{
  if(TMRS12_IsTimerExpired(4)) {
   printf("timer 4 expired \r\n");
   return 1;
  }
  return 0;
}

void RespToTmr4(SERVICE_PARAM)
{                       
  EnterWallTravelingState();
  TMRS12_ClearTimerExpired(4);
}

unsigned char TestForTmr5(EVENT_PARAM)
{
  if(TMRS12_IsTimerExpired(5)) {
   return 1;
  }
  return 0;
}

void RespToTmr5(SERVICE_PARAM)
{                       
  EnterOrientationState1();
  TMRS12_ClearTimerExpired(5);
}

unsigned char TestForKey(EVENT_PARAM)
{
  char theKey;
  unsigned char EventOccured = (unsigned char)kbhit();
  if (EventOccured)
  {
		theKey = (char)getchar();
	  SET_SHARED_BYTE_TO(theKey);
  }
  return EventOccured;
}

void RespToKey(SERVICE_PARAM){
  EnterOrientationState();
}

